
#include <windows.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

typedef struct
{
 char Name[64];
 void *Params;
} TSynthProgramInfo;

typedef TSynthProgramInfo *PSynthProgramInfo;
        
class VSSynthBase
{
public:	 
	VSSynthBase(int ASampleRate) {SampleRate=ASampleRate;};   
	virtual ~VSSynthBase() {};

	virtual void SetProgram(int NewValue) {};
	virtual void GetProgramName(int Index,char *Result) {Result[0]=0;};
	virtual int GetProgramCount() {return 0;};
	virtual int GetParamCount(int GroupIndex) {return 0;};
	virtual void GetParamLabel(int GroupIndex,int Index,char *Result) {};
	virtual void GetParamDisplay(int GroupIndex,int Index,char *Result) {};
	virtual void GetParamValue(int GroupIndex,int Index,float *Result) {};
	virtual void SetParamValue(int GroupIndex,int Index,float NewValue) {};
	virtual int GetGroupCount() {return 0;};
	virtual void GetGroupName(int Index,char *Result) {};
	virtual void NoteOn(int Note,int Velocity,int ScaleCents) {};
	virtual void NoteOff() {};
	virtual void LoadParams(void *Params,int ParamsLen) {};
	virtual void GetParamsInfo(int *ParamsLen) {*ParamsLen=0;};
	virtual void StoreParams(void *Params,int ParamsLen) {};
	virtual int GetBuffer(float *Buffer,int SampleCount) {memcpy(Buffer,0,SampleCount*sizeof(float));return 0;};
	virtual void InitializeProgram() {};
	virtual void GetFileExt(char *Result) {Result[0]=0;};
	virtual int SaveProgram(char *FileName) {strcpy(FileName,"Not supported");return -1;};
	virtual void GetProgramInfo(char *FileName,int *ProgramCount,int *ParamsLen) {*ProgramCount=0;*ParamsLen=0;};
	virtual int LoadProgram(char *FileName,PSynthProgramInfo Result) {strcpy(FileName,"Not supported");return -1;};
	virtual void SetSampleRate(int ASampleRate) {SampleRate=ASampleRate;};
    virtual bool HasEditor() {return false;};
	virtual void GetEditorSize(int *Width,int *Height) {};
	virtual void ShowEditor(HWND frame) {};
	virtual void CloseEditor() {};
protected:
	int SampleRate;  
};

typedef void (*TSynthSetProgram)(VSSynthBase *Instance,int NewValue);
typedef void (*TSynthGetProgramName)(VSSynthBase *Instance,int Index,char *Result);
typedef int (*TSynthGetProgramCount)(VSSynthBase *Instance);
typedef void (*TSynthSetNoteOff)(VSSynthBase *Instance,bool NewValue);
typedef int (*TSynthGetParamCount)(VSSynthBase *Instance,int GroupIndex);
typedef void (*TSynthGetParamLabel)(VSSynthBase *Instance,int GroupIndex,int Index,char *Result);
typedef void (*TSynthGetParamDisplay)(VSSynthBase *Instance,int GroupIndex,int Index,char *Result);
typedef void (*TSynthGetParamValue)(VSSynthBase *Instance,int GroupIndex,int Index,float *Result);
typedef void (*TSynthSetParamValue)(VSSynthBase *Instance,int GroupIndex,int Index,float NewValue);
typedef int (*TSynthGetGroupCount)(VSSynthBase *Instance);
typedef void (*TSynthGetGroupName)(VSSynthBase *Instance,int Index,char *Result);
typedef void (*TSynthNoteOn)(VSSynthBase *Instance,int Note,int Velocity,int ScaleCents);
typedef void (*TSynthLoadParams)(VSSynthBase *Instance,void *Params,int ParamsLen);
typedef void (*TSynthGetParamsInfo)(VSSynthBase *Instance,int *ParamsLen);
typedef void (*TSynthStoreParams)(VSSynthBase *Instance,void *Params,int ParamsLen);
typedef int (*TSynthGetBuffer)(VSSynthBase *Instance,float *Buffer,int SampleCount);
typedef void (*TSynthInitializeProgram)(VSSynthBase *Instance);
typedef void (*TSynthGetFileExt)(VSSynthBase *Instance,char *Result);
typedef int (*TSynthSaveProgram)(VSSynthBase *Instance,char *FileName);
typedef void (*TSynthGetProgramInfo)(VSSynthBase *Instance,char *FileName,int *ProgramCount,int *ParamsLen);
typedef int (*TSynthLoadProgram)(VSSynthBase *Instance,char *FileName,PSynthProgramInfo Result);
typedef void (*TSynthSetSampleRate)(VSSynthBase *Instance,int SampleRate);
typedef bool (*TSynthHasEditor)(VSSynthBase *Instance);
typedef void (*TSynthGetEditorSize)(VSSynthBase *Instance,int *Width,int *Height);
typedef void (*TSynthShowEditor)(VSSynthBase *Instance,HWND frame);
typedef void (*TSynthCloseEditor)(VSSynthBase *Instance);

typedef struct 
{
 char Name[64];
 int Flags; //currently 0

 TSynthSetProgram SetProgram;
 TSynthGetProgramName GetProgramName;
 TSynthGetProgramCount GetProgramCount;
 TSynthSetNoteOff SetNoteOff;
 TSynthGetParamCount GetParamCount;
 TSynthGetParamLabel GetParamLabel;
 TSynthGetParamDisplay GetParamDisplay;
 TSynthGetParamValue GetParamValue;
 TSynthSetParamValue SetParamValue;
 TSynthGetGroupCount GetGroupCount;
 TSynthGetGroupName GetGroupName;
 TSynthNoteOn NoteOn;
 TSynthLoadParams LoadParams;
 TSynthGetParamsInfo GetParamsInfo;
 TSynthStoreParams StoreParams;
 TSynthGetBuffer GetBuffer;
 TSynthInitializeProgram InitializeProgram;
 TSynthGetFileExt GetFileExt;
 TSynthSaveProgram SaveProgram;
 TSynthGetProgramInfo GetProgramInfo;
 TSynthLoadProgram LoadProgram;
 TSynthSetSampleRate SetSampleRate;
 TSynthHasEditor HasEditor;
 TSynthGetEditorSize GetEditorSize;
 TSynthShowEditor ShowEditor;
 TSynthCloseEditor CloseEditor;
} TPlugInSynth;

typedef TPlugInSynth *PPlugInSynth;

//interface functions
void SynthSetProgram(VSSynthBase *Instance,int NewValue)
{
 Instance->SetProgram(NewValue);
}

void SynthGetProgramName(VSSynthBase *Instance,int Index,char *Result)
{
 Instance->GetProgramName(Index,Result);	 
}

int  SynthGetProgramCount(VSSynthBase *Instance)
{
 return Instance->GetProgramCount();
}

void SynthSetNoteOff(VSSynthBase *Instance,bool NewValue)
{
 if (NewValue) Instance->NoteOff();
}

int  SynthGetParamCount(VSSynthBase *Instance,int GroupIndex)
{
 return Instance->GetParamCount(GroupIndex);
}

void SynthGetParamLabel(VSSynthBase *Instance,int GroupIndex,int Index,char *Result)
{
 Instance->GetParamLabel(GroupIndex,Index,Result);
}

void SynthGetParamDisplay(VSSynthBase *Instance,int GroupIndex,int Index,char *Result)
{
 Instance->GetParamDisplay(GroupIndex,Index,Result);
}

void SynthGetParamValue(VSSynthBase *Instance,int GroupIndex,int Index,float *Result)
{
 Instance->GetParamValue(GroupIndex,Index,Result);
}

void SynthSetParamValue(VSSynthBase *Instance,int GroupIndex,int Index,float NewValue)
{
 Instance->SetParamValue(GroupIndex,Index,NewValue);
}

int  SynthGetGroupCount(VSSynthBase *Instance)
{
 return Instance->GetGroupCount();
}

void SynthGetGroupName(VSSynthBase *Instance,int Index,char *Result)
{
 Instance->GetGroupName(Index,Result);
}

void SynthNoteOn(VSSynthBase *Instance,int Note,int Velocity,int ScaleCents)
{
 Instance->NoteOn(Note,Velocity,ScaleCents);
}

void SynthLoadParams(VSSynthBase *Instance,void *Params,int ParamsLen)
{
 Instance->LoadParams(Params,ParamsLen);
}

void SynthGetParamsInfo(VSSynthBase *Instance,int *ParamsLen)
{
 Instance->GetParamsInfo(ParamsLen);
}

void SynthStoreParams(VSSynthBase *Instance,void *Params,int ParamsLen)
{
 Instance->StoreParams(Params,ParamsLen);
}

int SynthGetBuffer(VSSynthBase *Instance,float *Buffer,int SampleCount)
{
 return Instance->GetBuffer(Buffer,SampleCount);
}

void SynthInitializeProgram(VSSynthBase *Instance)
{
 Instance->InitializeProgram();
}

void SynthGetFileExt(VSSynthBase *Instance,char *Result)
{
 Instance->GetFileExt(Result);
}

int SynthSaveProgram(VSSynthBase *Instance,char *FileName)
{
 return Instance->SaveProgram(FileName);
}

void SynthGetProgramInfo(VSSynthBase *Instance,char *FileName,int *ProgramCount,int *ParamsLen)
{
 Instance->GetProgramInfo(FileName,ProgramCount,ParamsLen);
}

int SynthLoadProgram(VSSynthBase *Instance,char *FileName,PSynthProgramInfo Result)
{
 return Instance->LoadProgram(FileName,Result);
}

void SynthSetSampleRate(VSSynthBase *Instance,int SampleRate)
{
 Instance->SetSampleRate(SampleRate);
}

bool SynthHasEditor(VSSynthBase *Instance)
{
 return Instance->HasEditor();
}

void SynthGetEditorSize(VSSynthBase *Instance,int *Width,int *Height)
{
 Instance->GetEditorSize(Width,Height);
}

void SynthShowEditor(VSSynthBase *Instance,HWND frame)
{
 Instance->ShowEditor(frame);
}

void SynthCloseEditor(VSSynthBase *Instance)
{
 Instance->CloseEditor();
}


static TPlugInSynth SynthParams=
{
 "",
 0, //currently 0
 SynthSetProgram,
 SynthGetProgramName,
 SynthGetProgramCount,
 SynthSetNoteOff,
 SynthGetParamCount,
 SynthGetParamLabel,
 SynthGetParamDisplay,
 SynthGetParamValue,
 SynthSetParamValue,
 SynthGetGroupCount,
 SynthGetGroupName,
 SynthNoteOn,
 SynthLoadParams,
 SynthGetParamsInfo,
 SynthStoreParams,
 SynthGetBuffer,
 SynthInitializeProgram,
 SynthGetFileExt,
 SynthSaveProgram,
 SynthGetProgramInfo,
 SynthLoadProgram,
 SynthSetSampleRate,
 SynthHasEditor,
 SynthGetEditorSize,
 SynthShowEditor,
 SynthCloseEditor
};

//Helpers

char *Float2String(float value)
{
 static char Result[255];
 sprintf(Result,"%.2f",value);
 return (char *)&Result;
}

char *Int2String(int value)
{
 static char Result[255];
 sprintf(Result,"%i",value);
 return (char *)&Result;
}

